"""Test the functionality of conf2spec part of the tool.
conf2spec can do a bit more that the fully automatic generation of spec files.
The tests test both the automatically generated files and the custom
configurations.
"""

from pathlib import Path
import pytest

from pyp2spec import conf2spec


def get_config_files(prefix):
    """Yield the relative paths to the config files placed in
    'tests/test_configs'."""

    source_path = Path("tests/test_configs")
    for path in source_path.iterdir():
        if path.name.startswith(prefix):
            yield path


@pytest.mark.parametrize(("config_file"), get_config_files("customized"))
def test_custom_generated_specfile(file_regression, config_file):
    # Run the conf2spec converter
    rendered_file = conf2spec.create_spec_file(config_file)

    # Compare the results
    with open(rendered_file, "r") as rendered_f:
        rendered = rendered_f.read()

    try:
        file_regression.check(
            rendered,
            fullpath=f"tests/expected_specfiles/customized/{rendered_file}",
        )

    finally:
        # Cleanup - remove created file
        Path.unlink(Path(rendered_file))


@pytest.mark.parametrize(("config_file"), get_config_files("default"))
def test_default_generated_specfile(file_regression, config_file):
    # Run the conf2spec converter
    rendered_file = conf2spec.create_spec_file(config_file)

    # Compare the results
    with open(rendered_file, "r") as rendered_f:
        rendered = rendered_f.read()

    try:
        file_regression.check(
            rendered,
            fullpath=f"tests/expected_specfiles/default/{rendered_file}",
        )

    finally:
        # Cleanup - remove created file
        Path.unlink(Path(rendered_file))


from datetime import date
from importlib.resources import open_text
from subprocess import check_output
import json

import click
import requests
import tomli_w


class PypiPackage:
    """Get and save package data from PyPI."""

    def __init__(self, package, session=None):
        self.package = package
        self.package_data = self.get_package_metadata(session)

    @property
    def pypi_name(self):
        return self.package_data["info"]["name"]

    def get_package_metadata(self, session):
        pkg_index = f"https://pypi.org/pypi/{self.package}/json"
        s = session or requests.Session()
        response = s.get(pkg_index)
        return response.json()

    def python_name(self):
        if self.pypi_name.startswith("python"):
            return self.pypi_name
        else:
            return f"python-{self.pypi_name}"

    def source_url(self, version):
        return "%{pypi_source " + self.archive_name(version) + "}"

    def version(self):
        return self.package_data["info"]["version"]

    def license(self, compliant=False):
        click.secho(f"Attempting to get license from Classifiers", fg='cyan')
        self.classifiers = self.read_license_classifiers()
        # Process classifiers further if there are some
        if self.classifiers:
            pkg_license = self.get_license_from_classifiers(compliant)
        else:
            click.secho(f"License in Classifiers not found, looking for the 'License' keyword", fg='cyan')
            pkg_license = self.package_data["info"]["license"]

        if pkg_license:
            return pkg_license
        else:
            click.secho(f"License not found. Specify --license explicitly. Quitting", fg='red')
            exit(1)

    def get_license_from_classifiers(self, compliant):
        license_map = self.read_license_map()
        licenses = []
        for classifier in self.classifiers:
            short_license, fedora_status = license_map[classifier]
            if compliant:
                # "???" are APSL, Artistic, Eiffel - some versions are Fedora OK,
                # some not. On PyPI there are <100 packages with them, rather than
                # adding another layer of decision matrix, just skip them all.
                if fedora_status in ["BAD", "UNKNOWN", "???"]:
                    click.secho(f"License '{short_license}' is or may not be allowed in Fedora, quitting", fg='red')
                    exit(1)
                else:
                    licenses.append(short_license)
            else:
                licenses.append(short_license)
        return " and ".join(licenses)

    def read_license_map(self):
        with open_text("pyp2spec", "classifiers_to_fedora.json") as f:
            license_map = json.load(f)
        return license_map

    def read_license_classifiers(self):
        classifiers = self.package_data["info"]["classifiers"]
        return [c for c in classifiers if c.startswith("License")]

    def summary(self):
        return self.package_data["info"]["summary"]

    def project_url(self):
        try:
            return self.package_data["info"]["project_urls"]["Homepage"]
        # It may happen that no Homepage is listed with the project
        # In this case fall back to the safe PyPI URL
        except KeyError:
            return self.package_data["info"]["package_url"]

    def archive_name(self, version):
        for entry in self.package_data["releases"][version]:
            if entry["packagetype"] == "sdist":
                return "-".join(entry["filename"].split("-")[:-1])
        else:
            click.secho(f"sdist not found, quitting", fg='red')
            exit(1)


def changelog_head(email, name, changelog_date):
    """Return f'{date} {name} <{email}>'"""

    # Date is set for the tests
    if not changelog_date:
        changelog_date = (date.today()).strftime("%a %b %d %Y")

    if email or name:
        return f"{changelog_date} {name} <{email}>"

    try:
        result = check_output(["rpmdev-packager"])
        result = result.decode().strip()
    except FileNotFoundError:
        # Set a dummy value not to fail on missing changelog data
        result = "mockbuilder"

    return f"{changelog_date} {result}"


def changelog_msg():
    """Return a default changelog message."""

    return "Package generated with pyp2spec"


def get_description(package):
    """Return a default package description."""

    return f"This is package '{package}' generated automatically by pyp2spec."


def is_package_name(package):
    """Check whether the string given as package contains `/`.
    Return True if not, False otherwise."""

    if "/" in package:
        # It's probably URL
        click.secho(f"Assuming '{package}' is a URL", fg='yellow')
        return False
    click.secho(f"Assuming '{package}' is a package name", fg='yellow')
    return True


def create_config_contents(
    package,
    description=None,
    release=None,
    message=None,
    email=None,
    name=None,
    version=None,
    summary=None,
    date=None,
    session=None,
    license=None,
    compliant=False,
):
    """Use `package` and provided kwargs to create the whole config contents.
    Return contents dictionary.
    """
    contents = {}

    # a package name was given as the `package`, look for it on PyPI
    if is_package_name(package):
        pkg = PypiPackage(package, session)
        click.secho(f"Querying PyPI for package '{package}'", fg='cyan')
    # a URL was given as the `package`
    else:
        raise NotImplementedError

    # If the arguments weren't provided via CLI,
    # get them from the stored package object data or the default values
    if message is None:
        message = changelog_msg()
        click.secho(f'Assuming changelog --message={message}', fg='yellow')

    if description is None:
        description = get_description(package)
        click.secho(f'Assuming --description={description}', fg='yellow')

    if summary is None:
        summary = pkg.summary()
        click.secho(f'Assuming --summary={summary}', fg='yellow')

    if version is None:
        version = pkg.version()
        click.secho(f'Assuming --version={version}', fg='yellow')

    if license is None:
        license = pkg.license(compliant)
        click.secho(f'Assuming --license={license}', fg='yellow')

    if release is None:
        release = "1"
        click.secho(f'Assuming --release={release}', fg='yellow')

    contents["changelog_msg"] = message
    contents["changelog_head"] = changelog_head(email, name, date)
    contents["description"] = description
    contents["summary"] = summary
    contents["version"] = version
    contents["license"] = license
    contents["release"] = release
    contents["pypi_name"] = pkg.pypi_name
    contents["python_name"] = pkg.python_name()
    contents["url"] = pkg.project_url()
    contents["source"] = pkg.source_url(version)
    contents["archive_name"] = pkg.archive_name(version)

    return contents


def save_config(contents, output=None):
    """Write config file to a given destination.
    If none is provided, save it to current directory with package name as file name.
    Return the saved file name.
    """
    if not output:
        package = contents["python_name"]
        output = f"./{package}.conf"
    with open(output, "wb") as f:
        click.secho(f"Saving configuration file to '{output}'", fg='yellow')
        tomli_w.dump(contents, f, multiline_strings=True)
    click.secho("Configuration file was saved successfully", fg="green")
    return output


def create_config(options):
    """Create and save config file."""

    contents = create_config_contents(
        options["package"],
        description=options["description"],
        release=options["release"],
        message=options["message"],
        email=options["email"],
        name=options["packager"],
        version=options["version"],
        summary=options["summary"],
        license=options["license"],
        compliant=options["fedora_compliant"]
    )
    return save_config(contents, options["config_output"])


@click.command()
@click.argument("package")
@click.option(
    "--config-output", "-c",
    help="Provide custom output for configuration file",
)
@click.option(
    "--description", "-d",
    help="Provide description for the package",
)
@click.option(
    "--release", "-r",
    help="Provide Fedora release, default: 1",
)
@click.option(
    "--message", "-m",
    help="Provide custom changelog message for the package",
)
@click.option(
    "--email", "-e",
    help="Provide e-mail for changelog, default: output of `rpmdev-packager`",
)
@click.option(
    "--packager", "-p",
    help="Provide packager name for changelog, default: output of `rpmdev-packager`",
)
@click.option(
    "--version", "-v",
    help="Provide package version to query PyPI for, default: latest",
)
@click.option(
    "--summary", "-s",
    help="Provide custom package summary",
)
@click.option(
    "--license", "-l",
    help="Provide license name",
)
@click.option(
    "--fedora-compliant", is_flag=True,
    help="Check whether license is compliant with Fedora",
)
def main(**options):
    create_config(options)


if __name__ == "__main__":
    main()



from pyp2spec.pyp2conf import create_config
from pyp2spec.conf2spec import create_spec_file
import click


@click.command()
@click.argument("package")
@click.option(
    "--config-output", "-c",
    help="Provide custom output for configuration file",
)
@click.option(
    "--description", "-d",
    help="Provide description for the package",
)
@click.option(
    "--release", "-r",
    help="Provide Fedora release, default: 1",
)
@click.option(
    "--message", "-m",
    help="Provide custom changelog message for the package",
)
@click.option(
    "--email", "-e",
    help="Provide e-mail for changelog, default: output of `rpmdev-packager`",
)
@click.option(
    "--packager", "-p",
    help="Provide packager name for changelog, default: output of `rpmdev-packager`",
)
@click.option(
    "--version", "-v",
    help="Provide package version to query PyPI for, default: latest",
)
@click.option(
    "--summary", "-s",
    help="Provide custom package summary",
)
@click.option(
    "--license", "-l",
    help="Provide license name",
)
@click.option(
    "--spec-output", "-s",
    help="Provide custom output where spec file will be saved",
)
@click.option(
    "--fedora-compliant", is_flag=True,
    help="Check whether license is compliant with Fedora",
)
def main(**options):
    click.secho("Generating configuration file", fg="cyan")
    config_file = create_config(options)
    click.secho("Generating spec file", fg="cyan")
    create_spec_file(config_file, options["spec_output"])
    click.secho("Done", fg="green")


if __name__ == "__main__":
    main()



from importlib.resources import read_text

import click
import tomli

from jinja2 import Template


TEMPLATE_FILENAME = 'template.spec'


class ConfigFile:
    """Load configuration file and return its validated values."""

    def __init__(self, filename):
        self.filename = filename
        self.contents = self.load_configuration

    @property
    def load_configuration(self):
        """Return loaded TOML configuration file contents."""

        with open(self.filename, "rb") as configuration_file:
            loaded_contents = tomli.load(configuration_file)

        return loaded_contents

    def get_string(self, key):
        """Return a value for given key. Validate the value is a string.
        Raise TypeError otherwise."""

        return self._get_value(key, str)

    def get_list(self, key):
        """Return a value for given key. Validate the value is a list.
        Raise TypeError otherwise."""

        return self._get_value(key, list)

    def _get_value(self, key, val_type):
        val = self.contents.get(key, val_type())
        if not isinstance(val, val_type):
            raise TypeError(f"{val} must be a {val_type}")
        return val


def generate_extra_build_requires(config):
    """If defined in config file, return extra build requires.
    If none were defined, return `-r` - runtime extra BRs`."""

    # TODO: custom tox env (-e)
    options = {
        "tox": "-t",
        "extra": "-x",
    }
    extra_brs = config.get_list("extra_build_requires")

    # No extra BuildRequires were defined - return `-r` = runtime
    if not extra_brs:
        return "-r"

    generated_brs = []
    add = generated_brs.append
    for extra_br in extra_brs:
        if extra_br == "extra":
            add(options.get(extra_br))
            add(",".join(config.get_list("extra_test_env")))
        else:
            add(options.get(extra_br))

    return " ".join(generated_brs)


def generate_check(config):
    """Generate valid check section.
    If defined in config file, use applicable test macro.
    If no test method was defined, return an empty string."""

    test_method = config.get_string("test_method")
    if test_method == "pytest":
        return generate_pytest(config)
    elif test_method == "tox":
        return generate_tox(config)
    else:
        return ""


def generate_pytest(config):
    """Return valid pytest macro with unwanted tests (if defined)."""

    if unwanted_tests := generate_unwanted_tests(config):
        return f"%pytest {unwanted_tests}"
    return "%pytest"


def generate_tox(config):
    """Return valid tox macro. If additional unwanted are defined,
    raise NotImplementedError."""

    if unwanted_tests := generate_unwanted_tests(config):
        raise NotImplementedError
    return "%tox"


def generate_unwanted_tests(config):
    """If defined in config file, get the unwanted tests.
    Given the unwanted tests are [a, b], return `-k "not a and\\\nnot b"`."""

    unwanted_tests = config.get_list("unwanted_tests")
    if not unwanted_tests:
        return ""
    else:
        prep_unwanteds = [f"not {test}" for test in unwanted_tests]
        unwanteds_as_str = " and \\\n".join(prep_unwanteds)
        formatted_unwanteds = f'-k "{unwanteds_as_str}"'
        return formatted_unwanteds


def fill_in_template(config):
    """Return template rendered with data from config file."""

    spec_template = Template(read_text("pyp2spec", TEMPLATE_FILENAME))

    result = spec_template.render(
        archive_name=config.get_string("archive_name"),
        binary_files=config.get_list("binary_files"),
        changelog_head=config.get_string("changelog_head"),
        changelog_msg=config.get_string("changelog_msg"),
        description=config.get_string("description"),
        doc_files=" ".join(config.get_list("doc_files")),
        extra_build_requires=generate_extra_build_requires(config),
        license_files=" ".join(config.get_list("license_files")),
        license=config.get_string("license"),
        manual_build_requires=config.get_list("manual_build_requires"),
        name=config.get_string("pypi_name"),
        python_name=config.get_string("python_name"),
        release=config.get_string("release"),
        source=config.get_string("source"),
        summary=config.get_string("summary"),
        test_method=generate_check(config),
        url=config.get_string("url"),
        version=config.get_string("version"),
    )

    return result


def save_spec_file(config, output):
    """Save the spec file in the current directory if custom output is not set.
    Return the saved file name."""

    result = fill_in_template(config)
    if output is None:
        output = config.get_string("python_name") + ".spec"
    with open(output, "w") as spec_file:
        click.secho(f"Saving spec file to '{output}'", fg='yellow')
        spec_file.write(result)
    click.secho("Spec file was saved successfully", fg="green")
    return output


def create_spec_file(config_file, spec_output=None):
    """Create and save the generate spec file."""
    config = ConfigFile(config_file)
    return save_spec_file(config, spec_output)


@click.command()
@click.argument("config")
@click.option(
    "--spec-output",
    "-s",
    help="Provide custom output for spec file",
)
def main(config, spec_output):
    create_spec_file(config, spec_output)


if __name__ == "__main__":
    main()



import betamax
import click
import pytest
import tomli

from pyp2spec.pyp2conf import PypiPackage, create_config_contents


with betamax.Betamax.configure() as config:
    config.cassette_library_dir = 'tests/fixtures/cassettes'


@pytest.fixture
def changelog():
    return ("Wed Nov 03 2021", "Packager", "packager@maint.com")


@pytest.mark.parametrize("package, version",
    [
        ("aionotion", "2.0.3"),
        ("aioflo", "0.4.2"),
        ("click", "7.1.2"),
        ("jupyter-packaging", "0.10.4"),
        ("markdown-it-py", "1.1.0"),
    ]
)
def test_automatically_generated_config_is_valid(betamax_parametrized_session, changelog, package, version):
    """Run the config rendering in fully automated mode and compare the results"""
    config = create_config_contents(
        package=package,
        version=version,
        date=changelog[0],
        name=changelog[1],
        email=changelog[2],
        session=betamax_parametrized_session,
    )

    with open(f"tests/test_configs/default_python-{package}.conf", "rb") as config_file:
        loaded_contents = tomli.load(config_file)

    assert config == loaded_contents


def test_config_with_customization_is_valid(betamax_session):
    """Get the upstream metadata and modify some fields to get the custom config file.
    """
    package = "aionotion"
    config = create_config_contents(
        package=package,
        description="A asyncio-friendly library for Notion Home Monitoring devices.\n",
        release="4",
        message="Rebuilt for Python 3.10",
        email="package@manager.com",
        name="Package Manager",
        version="2.0.3",
        summary="Python library for Notion Home Monitoring",
        date="Fri Jun 04 2021",
        session=betamax_session,
    )

    with open(f"tests/test_configs/customized_{package}.conf", "rb") as config_file:
        loaded_contents = tomli.load(config_file)

    assert config == loaded_contents


def test_license_classifier_read_correctly(betamax_session):
    pkg = PypiPackage("tomli")
    assert pkg.read_license_classifiers() == ['License :: OSI Approved :: MIT License']


def test_no_license_classifiers_and_no_license_keyword(betamax_session):
    pkg = PypiPackage("tomli")
    pkg.package_data["info"]["classifiers"] = []
    assert pkg.read_license_classifiers() == []
    with pytest.raises(SystemExit):
        pkg.license()


def test_compliant_license_is_returned(betamax_session):
    pkg = PypiPackage("tomli")
    pkg.classifiers = pkg.read_license_classifiers()
    assert pkg.get_license_from_classifiers(compliant=True) == "MIT"


def test_bad_license_if_compliant_is_not_returned(betamax_session):
    pkg = PypiPackage("tomli")
    pkg.classifiers = ["License :: Eiffel Forum License (EFL)"]
    with pytest.raises(SystemExit):
        pkg.get_license_from_classifiers(compliant=True)


def test_bad_license_if_not_compliant_is_returned(betamax_session):
    pkg = PypiPackage("tomli")
    pkg.classifiers = ["License :: Eiffel Forum License (EFL)"]
    pkg.get_license_from_classifiers(compliant=False) == "EFL"


def test_mix_non_compliant_licenses_wont_work_if_strict(betamax_session):
    pkg = PypiPackage("tomli")
    pkg.classifiers = [
        "License :: Eiffel Forum License (EFL)",
        "License :: OSI Approved :: MIT License",
    ]
    with pytest.raises(SystemExit):
        pkg.get_license_from_classifiers(compliant=True)


def test_mix_non_compliant_licenses_work_if_not_strict(betamax_session):
    pkg = PypiPackage("tomli")
    pkg.classifiers = [
        "License :: Eiffel Forum License (EFL)",
        "License :: OSI Approved :: MIT License",
    ]
    pkg.get_license_from_classifiers(compliant=False) == "EFL and MIT"